/* * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * */
/* CVC Code Generation for Automatic Generation of Attack Format Strings     */
/*                                                                           */
/* Vinod Ganapathy                                       6 November 2003     */
/*                                                                           */
/* Functionality: Given the set of attack parameters, automatically generate */
/* CVC code for the format string attack.                                    */
/*                                                                           */
/* The code in this file is specifically targeted at printf, and the attack  */
/* that reads from an arbitrary location in memory.                          */
/* * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * */

#ifndef PROBLEM_INSTANCE_H
#define PROBLEM_INSTANCE_H

#include <assert.h>
#include <iostream>
#include <fstream>
#include <string>
#include <set>

class printf_read_attack
{
  
private:
  
  // The length of the format string, which is an attack parameter.
  int fmt_length;

  // The distance from the end of pointer to the format string to the 
  // beginning of the format string. Hence, this attack parameter is
  // assumed not to include the length of the pointer to the format
  // string itself.
  int distance;

  // The address is a 4 byte value. Since we do not want to get into
  // the low level details (using hexadecimal etc.) in the CVC implementation
  // we shall represent these addresses using 4 integers between 0-255 to
  // represent the 4 bytes.
  // Note: Little endian => LSB stored in a lower address.

  int address_lsb, address_med_lsb, address_med_msb, address_msb;
  
  // The format characters that are used in the attack. This is not an attack
  // parameter, but is used at several places.
  std::set<std::string> fc_set;
  
   // Declarations of the variables
  void declarations (void);

  // Initialization code (assignment of symbols to ASCII values),
  // and concretization of the attack parameters
  void initialization(void);
  
  // Attack Encoding. The actual set of boolean constraints that are
  // crucial to encode this attack.
  void attack_encoding (void);
  
  // Restrict the contents of the format string to a particular alphabet. 
  void format_string_alphabet (void);
  
  // Counter Definition.
  void counter_definition (void);

  // Integrality Constraints.
  void integrality_constraints (void);

  // Query String.
  void query (void);

public:
  printf_read_attack (int a_fmt_length, 
                      int a_distance, 
                      int a_address_lsb, 
                      int a_address_med_lsb, 
                      int a_address_med_msb, 
                      int a_address_msb) {
    fmt_length = a_fmt_length;
    distance = a_distance;
    address_lsb = a_address_lsb;
    address_med_lsb = a_address_med_lsb;
    address_med_msb = a_address_med_msb;
    address_msb = a_address_msb;
  }

  void generate_cvc_code(void);

};

// TODO
// 1. Finish implementation of current thing in cpp
// 2. Investigate the use of the API.
// 3. Add more directives.
#endif
